class PeachException(Exception):
    pass


class IntermittentException(PeachException):
    """Worth retrying

    e.g. database connection dropped, lock contention, timeouts."""

    pass


class CodeException(PeachException):
    """Not worth retrying until new code has been deployed

    e.g. code syntax error."""

    pass


class IncompatibleSchemaException(PeachException):
    """Not worth retrying until new code has been deployed

    e.g. mistmatch in field names or types."""

    pass


class UpstreamDataException(PeachException):
    """Not worth retrying until upstream data has been rebuilt

    e.g. column missing, contains invalid/unexpected values, etc.

    Not guaranteed that fixing upstream data is enough, e.g. if unexpected
    data is actually valid, then a code update is required.
    """

    pass


class ProjectStructureException(PeachException):
    """Should not retry until project structure is fixed

    e.g. missing manifest files.
    """

    pass


class CLIArgumentException(PeachException):
    """Should not retry until CLI argument is fixed

    e.g. invalid value for enum.
    """

    pass


class ConfigException(PeachException):
    """Should not retry until config is fixed

    e.g. missing container registry for tasks run in the cloud.
    """

    pass


class CommandException(PeachException):
    """Should not retry until command is fixed

    e.g. cannot install conflicting dependencies inside docker image.
    """
