from dataclasses import dataclass
import datetime
import inspect
import logging
from pathlib import Path
from typing import Dict, List

import pyarrow as pa

from peach.constants import Mode
from peach.datastores import Datastore
from peach.jobs import Job
from peach.table import Table, TableKind, TABLES_DIR


@dataclass
class SharedState:
    datastore: Datastore
    tables: Dict[str, Table]
    jobs: Dict[str, Job]
    mode: Mode
    default_datastore_id: str = None
    docker_repository: str = None
    dev_id: str = None

    def __init__(self):
        self.tables: Dict[str, Table] = {}
        self.jobs: Dict[str, Job] = {}
        self.datastore: Datastore = None

    def job(self, table: Table):
        return self.jobs[table.id]

    def declare_datastore(self, datastore: Datastore):
        assert self.datastore is None
        self.datastore = datastore

    def declare_job(self, job: Job):
        source_path = Path(inspect.stack()[1].filename).resolve()
        ds_dir_pos = [i for i, x in enumerate(source_path.parts) if x == TABLES_DIR][-1]
        id = str(Path(*source_path.parts[ds_dir_pos + 1 : -1]))

        logging.info(f"declaring job: {id}")
        assert id not in self.jobs
        self.jobs[id] = job

    def declare_table(
        self,
        schema: pa.Schema = None,
        kind: TableKind = None,
        inputs: List[str] = None,
        refresh_rate: datetime.timedelta = None,
        is_external=False,
    ):
        inputs = inputs or []

        source_path = Path(inspect.stack()[1].filename).resolve()
        ds_dir_pos = [i for i, x in enumerate(source_path.parts) if x == TABLES_DIR][-1]
        id = str(Path(*source_path.parts[ds_dir_pos + 1 : -1]))

        logging.info(f"delcaring table: {id}")
        assert id not in self.tables
        self.tables[id] = Table(
            id=id,
            schema=schema,
            kind=kind,
            inputs=inputs,
            refresh_rate=refresh_rate,
            is_external=is_external,
        )

    def input_tables(self, table: Table) -> Dict[str, Table]:
        return {self.tables[t].id: self.tables[t] for t in table.inputs}


shared_state = SharedState()
